@@ -38,11 +38,15 @@
 #define WP_PATH_ID "org.gnome.desktop.background"
 #define WP_LOCK_PATH_ID "org.gnome.desktop.screensaver"
 #define WP_URI_KEY "picture-uri"
+#define WP_URI_DARK_KEY "picture-uri-dark"
 #define WP_OPTIONS_KEY "picture-options"
 #define WP_SHADING_KEY "color-shading-type"
 #define WP_PCOLOR_KEY "primary-color"
 #define WP_SCOLOR_KEY "secondary-color"
 
+#define INTERFACE_PATH_ID "org.gnome.desktop.interface"
+#define INTERFACE_COLOR_SCHEME_KEY "color-scheme"
+
 struct _CcBackgroundPanel
 {
   CcPanel parent_instance;
@@ -51,25 +55,135 @@ struct _CcBackgroundPanel
 
   GSettings *settings;
   GSettings *lock_settings;
+  GSettings *interface_settings;
 
   GnomeDesktopThumbnailFactory *thumb_factory;
+  GDBusProxy *proxy;
 
   CcBackgroundItem *current_background;
 
   CcBackgroundChooser *background_chooser;
-  GtkButton *add_picture_button;
-  CcBackgroundPreview *desktop_preview;
+  CcBackgroundPreview *light_preview;
+  CcBackgroundPreview *dark_preview;
+  GtkToggleButton *light_toggle;
+  GtkToggleButton *dark_toggle;
 };
 
 CC_PANEL_REGISTER (CcBackgroundPanel, cc_background_panel)
 
+static void
+load_custom_css (CcBackgroundPanel *self)
+{
+  g_autoptr(GtkCssProvider) provider = NULL;
+
+  provider = gtk_css_provider_new ();
+  gtk_css_provider_load_from_resource (provider, "/org/gnome/control-center/background/preview.css");
+  gtk_style_context_add_provider_for_display (gdk_display_get_default (),
+                                              GTK_STYLE_PROVIDER (provider),
+                                              GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+}
+
+static void
+reload_light_dark_toggles (CcBackgroundPanel *self)
+{
+  GDesktopColorScheme scheme;
+
+  scheme = g_settings_get_enum (self->interface_settings, INTERFACE_COLOR_SCHEME_KEY);
+
+  if (scheme == G_DESKTOP_COLOR_SCHEME_DEFAULT)
+    {
+      gtk_toggle_button_set_active (self->light_toggle, TRUE);
+    }
+  else if (scheme == G_DESKTOP_COLOR_SCHEME_PREFER_DARK)
+    {
+      gtk_toggle_button_set_active (self->dark_toggle, TRUE);
+    }
+  else
+    {
+      gtk_toggle_button_set_active (self->light_toggle, FALSE);
+      gtk_toggle_button_set_active (self->dark_toggle, FALSE);
+    }
+}
+
+static void
+transition_screen (CcBackgroundPanel *self)
+{
+  g_autoptr (GError) error = NULL;
+
+  if (!self->proxy)
+    return;
+
+  g_dbus_proxy_call_sync (self->proxy,
+                          "ScreenTransition",
+                          NULL,
+                          G_DBUS_CALL_FLAGS_NONE,
+                          -1,
+                          NULL,
+                          &error);
+
+  if (error)
+    g_warning ("Couldn't transition screen: %s", error->message);
+}
+
+static void
+set_color_scheme (CcBackgroundPanel   *self,
+                  GDesktopColorScheme  color_scheme)
+{
+  GDesktopColorScheme scheme;
+
+  scheme = g_settings_get_enum (self->interface_settings,
+                                INTERFACE_COLOR_SCHEME_KEY);
+
+  /* We have to check the equality manually to avoid starting an unnecessary
+   * screen transition */
+  if (color_scheme == scheme)
+    return;
+
+  transition_screen (self);
+
+  g_settings_set_enum (self->interface_settings,
+                       INTERFACE_COLOR_SCHEME_KEY,
+                       color_scheme);
+}
+
+/* Color schemes */
+
+static void
+on_light_dark_toggle_active_cb (CcBackgroundPanel *self)
+{
+  if (gtk_toggle_button_get_active (self->light_toggle))
+    set_color_scheme (self, G_DESKTOP_COLOR_SCHEME_DEFAULT);
+  else if (gtk_toggle_button_get_active (self->dark_toggle))
+    set_color_scheme (self, G_DESKTOP_COLOR_SCHEME_PREFER_DARK);
+}
+
+static void
+got_transition_proxy_cb (GObject      *source_object,
+                         GAsyncResult *res,
+                         gpointer      data)
+{
+  g_autoptr(GError) error = NULL;
+  CcBackgroundPanel *self = data;
+
+  self->proxy = g_dbus_proxy_new_for_bus_finish (res, &error);
+
+  if (self->proxy == NULL)
+    {
+      g_warning ("Error creating proxy: %s", error->message);
+      return;
+    }
+}
+
+/* Background */
+
 static void
 update_preview (CcBackgroundPanel *panel)
 {
   CcBackgroundItem *current_background;
 
   current_background = panel->current_background;
-  cc_background_preview_set_item (panel->desktop_preview, current_background);
+  cc_background_preview_set_item (panel->light_preview, current_background);
+  cc_background_preview_set_item (panel->dark_preview, current_background);
 }
 
 static gchar *
@@ -89,6 +203,7 @@ reload_current_bg (CcBackgroundPanel *panel)
   CcBackgroundItem *configured;
   GSettings *settings = NULL;
   g_autofree gchar *uri = NULL;
+  g_autofree gchar *dark_uri = NULL;
   g_autofree gchar *pcolor = NULL;
   g_autofree gchar *scolor = NULL;
 
@@ -102,12 +217,15 @@ reload_current_bg (CcBackgroundPanel *panel)
   if (uri && *uri == '\0')
     g_clear_pointer (&uri, g_free);
 
+
   configured = cc_background_item_new (uri);
 
+  dark_uri = g_settings_get_string (settings, WP_URI_DARK_KEY);
   pcolor = g_settings_get_string (settings, WP_PCOLOR_KEY);
   scolor = g_settings_get_string (settings, WP_SCOLOR_KEY);
   g_object_set (G_OBJECT (configured),
                 "name", _("Current background"),
+                "uri-dark", dark_uri,
                 "placement", g_settings_get_enum (settings, WP_OPTIONS_KEY),
                 "shading", g_settings_get_enum (settings, WP_SHADING_KEY),
                 "primary-color", pcolor,
@@ -154,7 +272,8 @@ create_save_dir (void)
 static void
 set_background (CcBackgroundPanel *panel,
                 GSettings         *settings,
-                CcBackgroundItem  *item)
+                CcBackgroundItem  *item,
+                gboolean           set_dark)
 {
   GDesktopBackgroundStyle style;
   CcBackgroundItemFlags flags;
@@ -169,6 +288,18 @@ set_background (CcBackgroundPanel *panel,
 
   g_settings_set_string (settings, WP_URI_KEY, uri);
 
+  if (set_dark)
+    {
+      const char *uri_dark;
+
+      uri_dark = cc_background_item_get_uri_dark (item);
+
+      if (uri_dark && uri_dark[0])
+        g_settings_set_string (settings, WP_URI_DARK_KEY, uri_dark);
+      else
+        g_settings_set_string (settings, WP_URI_DARK_KEY, uri);
+    }
+
   /* Also set the placement if we have a URI and the previous value was none */
   if (flags & CC_BACKGROUND_ITEM_HAS_PLACEMENT)
     {
@@ -196,13 +327,12 @@ set_background (CcBackgroundPanel *panel,
     cc_background_xml_save (panel->current_background, filename);
 }
 
-
 static void
-on_chooser_background_chosen_cb (CcBackgroundPanel          *self,
-                                 CcBackgroundItem           *item)
+on_chooser_background_chosen_cb (CcBackgroundPanel *self,
+                                 CcBackgroundItem  *item)
 {
-  set_background (self, self->settings, item);
-  set_background (self, self->lock_settings, item);
+  set_background (self, self->settings, item, TRUE);
+  set_background (self, self->lock_settings, item, FALSE);
 }
 
 static void
@@ -217,20 +347,6 @@ cc_background_panel_get_help_uri (CcPanel *panel)
   return "help:gnome-help/look-background";
 }
 
-static void
-cc_background_panel_constructed (GObject *object)
-{
-  CcBackgroundPanel *self;
-  CcShell *shell;
-
-  self = CC_BACKGROUND_PANEL (object);
-  shell = cc_panel_get_shell (CC_PANEL (self));
-
-  cc_shell_embed_widget_in_header (shell, GTK_WIDGET (self->add_picture_button), GTK_POS_RIGHT);
-
-  G_OBJECT_CLASS (cc_background_panel_parent_class)->constructed (object);
-}
-
 static void
 cc_background_panel_dispose (GObject *object)
 {
@@ -238,7 +354,9 @@ cc_background_panel_dispose (GObject *object)
 
   g_clear_object (&panel->settings);
   g_clear_object (&panel->lock_settings);
+  g_clear_object (&panel->interface_settings);
   g_clear_object (&panel->thumb_factory);
+  g_clear_object (&panel->proxy);
 
   G_OBJECT_CLASS (cc_background_panel_parent_class)->dispose (object);
 }
@@ -265,16 +383,18 @@ cc_background_panel_class_init (CcBackgroundPanelClass *klass)
 
   panel_class->get_help_uri = cc_background_panel_get_help_uri;
 
-  object_class->constructed = cc_background_panel_constructed;
   object_class->dispose = cc_background_panel_dispose;
   object_class->finalize = cc_background_panel_finalize;
 
   gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/control-center/background/cc-background-panel.ui");
 
-  gtk_widget_class_bind_template_child (widget_class, CcBackgroundPanel, add_picture_button);
   gtk_widget_class_bind_template_child (widget_class, CcBackgroundPanel, background_chooser);
-  gtk_widget_class_bind_template_child (widget_class, CcBackgroundPanel, desktop_preview);
+  gtk_widget_class_bind_template_child (widget_class, CcBackgroundPanel, light_preview);
+  gtk_widget_class_bind_template_child (widget_class, CcBackgroundPanel, dark_preview);
+  gtk_widget_class_bind_template_child (widget_class, CcBackgroundPanel, light_toggle);
+  gtk_widget_class_bind_template_child (widget_class, CcBackgroundPanel, dark_toggle);
 
+  gtk_widget_class_bind_template_callback (widget_class, on_light_dark_toggle_active_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_chooser_background_chosen_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_add_picture_button_clicked_cb);
 }
@@ -303,10 +423,33 @@ cc_background_panel_init (CcBackgroundPanel *panel)
   panel->lock_settings = g_settings_new (WP_LOCK_PATH_ID);
   g_settings_delay (panel->lock_settings);
 
+  panel->interface_settings = g_settings_new (INTERFACE_PATH_ID);
+
   /* Load the background */
   reload_current_bg (panel);
   update_preview (panel);
 
   /* Background settings */
   g_signal_connect_object (panel->settings, "changed", G_CALLBACK (on_settings_changed), panel, G_CONNECT_SWAPPED);
+
+  /* Interface settings */
+  reload_light_dark_toggles (panel);
+
+  g_signal_connect_object (panel->interface_settings,
+                           "changed::" INTERFACE_COLOR_SCHEME_KEY,
+                           G_CALLBACK (reload_light_dark_toggles),
+                           panel,
+                           G_CONNECT_SWAPPED);
+
+  g_dbus_proxy_new_for_bus (G_BUS_TYPE_SESSION,
+                            G_DBUS_PROXY_FLAGS_NONE,
+                            NULL,
+                            "org.gnome.Shell",
+                            "/org/gnome/Shell",
+                            "org.gnome.Shell",
+                            NULL,
+                            got_transition_proxy_cb,
+                            panel);
+
+  load_custom_css (panel);
 }
