@@ -21,7 +21,7 @@
 
 struct _CcWifiConnectionRow
 {
-  GtkListBoxRow    parent_instance;
+  AdwActionRow     parent_instance;
 
   gboolean         constructed;
 
@@ -31,12 +31,12 @@ struct _CcWifiConnectionRow
   NMDeviceWifi    *device;
   GPtrArray       *aps;
   NMConnection    *connection;
+  gboolean         known_connection;
 
   GtkLabel        *active_label;
   GtkCheckButton  *checkbutton;
   GtkSpinner      *connecting_spinner;
   GtkImage        *encrypted_icon;
-  GtkLabel        *name_label;
   GtkButton       *options_button;
   GtkImage        *strength_icon;
 };
@@ -49,6 +49,7 @@ enum
   PROP_DEVICE,
   PROP_APS,
   PROP_CONNECTION,
+  PROP_KNOWN_CONNECTION,
   PROP_LAST
 };
 
@@ -63,7 +64,7 @@ typedef enum
   NM_AP_SEC_OWE
 } NMAccessPointSecurity;
 
-G_DEFINE_TYPE (CcWifiConnectionRow, cc_wifi_connection_row, GTK_TYPE_LIST_BOX_ROW)
+G_DEFINE_TYPE (CcWifiConnectionRow, cc_wifi_connection_row, ADW_TYPE_ACTION_ROW)
 
 static GParamSpec *props[PROP_LAST];
 
@@ -209,13 +210,16 @@ update_ui (CcWifiConnectionRow *self)
                                            name, ssid_str);
         }
 
-      gtk_label_set_markup (self->name_label, title);
+      adw_preferences_row_set_title (ADW_PREFERENCES_ROW (self), title);
     }
   else
     {
+      g_autofree char *title_escaped = NULL;
+
       ssid = nm_access_point_get_ssid (best_ap);
       title = nm_utils_ssid_to_utf8 (g_bytes_get_data (ssid, NULL), g_bytes_get_size (ssid));
-      gtk_label_set_text (self->name_label, title);
+      title_escaped = g_markup_escape_text (title, -1);
+      adw_preferences_row_set_title (ADW_PREFERENCES_ROW (self), title_escaped);
     }
 
   if (active_connection)
@@ -251,40 +255,36 @@ update_ui (CcWifiConnectionRow *self)
     }
 
   gtk_widget_set_visible (GTK_WIDGET (self->active_label), active);
-  gtk_widget_set_visible (GTK_WIDGET (self->options_button), active || connecting);
+  gtk_widget_set_visible (GTK_WIDGET (self->options_button), active || connecting || self->known_connection);
 
   if (security != NM_AP_SEC_UNKNOWN && security != NM_AP_SEC_NONE && security != NM_AP_SEC_OWE)
     {
-      const gchar *icon_path;
+      const gchar *icon_name = "lock-small-symbolic";
 
       gtk_widget_set_child_visible (GTK_WIDGET (self->encrypted_icon), TRUE);
       if (security == NM_AP_SEC_WEP)
 	{
-          icon_path = "/org/gnome/control-center/network/warning-small-symbolic.svg";
+          icon_name = "warning-small-symbolic";
 	  gtk_widget_set_tooltip_text (GTK_WIDGET (self->encrypted_icon), _("Insecure network (WEP)"));
 	}
       else if (security == NM_AP_SEC_WPA)
 	{
-          icon_path = "/org/gnome/control-center/network/lock-small-symbolic.svg";
           gtk_widget_set_tooltip_text (GTK_WIDGET (self->encrypted_icon), _("Secure network (WPA)"));
 	}
       else if (security == NM_AP_SEC_WPA2)
 	{
-          icon_path = "/org/gnome/control-center/network/lock-small-symbolic.svg";
           gtk_widget_set_tooltip_text (GTK_WIDGET (self->encrypted_icon), _("Secure network (WPA2)"));
 	}
 	  else if (security == NM_AP_SEC_SAE)
 	{
-          icon_path = "/org/gnome/control-center/network/lock-small-symbolic.svg";
           gtk_widget_set_tooltip_text (GTK_WIDGET (self->encrypted_icon), _("Secure network (WPA3)"));
 	}
       else
 	{
-          icon_path = "/org/gnome/control-center/network/lock-small-symbolic.svg";
           gtk_widget_set_tooltip_text (GTK_WIDGET (self->encrypted_icon), _("Secure network"));
 	}
 
-      gtk_image_set_from_resource (self->encrypted_icon, icon_path);
+      gtk_image_set_from_icon_name (self->encrypted_icon, icon_name);
     }
   else
     {
@@ -323,20 +323,7 @@ cc_wifi_connection_row_constructed (GObject *object)
   G_OBJECT_CLASS (cc_wifi_connection_row_parent_class)->constructed (object);
 
   /* Reparent the label into the checkbox */
-  if (self->checkable)
-    {
-      gtk_widget_set_visible (GTK_WIDGET (self->checkbutton), TRUE);
-      g_object_ref (self->name_label);
-      gtk_container_remove (GTK_CONTAINER (gtk_widget_get_parent (GTK_WIDGET (self->name_label))),
-                            GTK_WIDGET (self->name_label));
-      gtk_container_add (GTK_CONTAINER (self->checkbutton), GTK_WIDGET (self->name_label));
-      gtk_widget_show (GTK_WIDGET (self->name_label));
-      g_object_unref (self->name_label);
-    }
-  else
-    {
-      gtk_widget_set_visible (GTK_WIDGET (self->checkbutton), FALSE);
-    }
+  gtk_widget_set_visible (GTK_WIDGET (self->checkbutton), self->checkable);
 
   update_ui (CC_WIFI_CONNECTION_ROW (object));
 }
@@ -377,6 +364,10 @@ cc_wifi_connection_row_get_property (GObject    *object,
       g_value_set_object (value, self->connection);
       break;
 
+    case PROP_KNOWN_CONNECTION:
+      g_value_set_boolean (value, self->known_connection);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -423,6 +414,10 @@ cc_wifi_connection_row_set_property (GObject      *object,
       self->connection = g_value_dup_object (value);
       break;
 
+    case PROP_KNOWN_CONNECTION:
+      self->known_connection = g_value_get_boolean (value);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -459,7 +454,6 @@ cc_wifi_connection_row_class_init (CcWifiConnectionRowClass *klass)
   gtk_widget_class_bind_template_child (widget_class, CcWifiConnectionRow, connecting_spinner);
   gtk_widget_class_bind_template_child (widget_class, CcWifiConnectionRow, encrypted_icon);
   gtk_widget_class_bind_template_child (widget_class, CcWifiConnectionRow, options_button);
-  gtk_widget_class_bind_template_child (widget_class, CcWifiConnectionRow, name_label);
   gtk_widget_class_bind_template_child (widget_class, CcWifiConnectionRow, strength_icon);
 
   gtk_widget_class_bind_template_callback (widget_class, configure_clicked_cb);
@@ -489,6 +483,11 @@ cc_wifi_connection_row_class_init (CcWifiConnectionRowClass *klass)
                                                  NM_TYPE_CONNECTION,
                                                  G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS);
 
+  props[PROP_KNOWN_CONNECTION] = g_param_spec_boolean ("known-connection", "Known Connection",
+                                                "Whether this row is a known connection or not",
+                                                FALSE,
+                                                G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS);
+
   g_object_class_install_properties (object_class,
                                      PROP_LAST,
                                      props);
@@ -523,13 +522,15 @@ CcWifiConnectionRow *
 cc_wifi_connection_row_new (NMDeviceWifi  *device,
                             NMConnection  *connection,
                             GPtrArray     *aps,
-                            gboolean       checkable)
+                            gboolean       checkable,
+                            gboolean       known_connection)
 {
   return g_object_new (CC_TYPE_WIFI_CONNECTION_ROW,
                        "device", device,
                        "connection", connection,
                        "aps", aps,
                        "checkable", checkable,
+                       "known-connection", known_connection,
                        NULL);
 }
 
