@@ -72,7 +72,7 @@ struct _PpPrinterEntry
   GtkWidget      *show_jobs_dialog_button;
   GtkWidget      *clean_heads_menuitem;
   GtkCheckButton *printer_default_checkbutton;
-  GtkModelButton *remove_printer_menuitem;
+  GtkWidget      *remove_printer_menuitem;
   GtkBox         *printer_error;
   GtkLabel       *error_status;
 
@@ -93,12 +93,6 @@ struct _PpPrinterEntryClass
 
 G_DEFINE_TYPE (PpPrinterEntry, pp_printer_entry, GTK_TYPE_LIST_BOX_ROW)
 
-enum {
-  PROP_0,
-  PROP_PRINTER_NAME,
-  PROP_PRINTER_LOCATION,
-};
-
 enum {
   IS_DEFAULT_PRINTER,
   PRINTER_DELETE,
@@ -124,49 +118,6 @@ ink_level_data_free (InkLevelData *data)
   g_slice_free (InkLevelData, data);
 }
 
-static void
-pp_printer_entry_get_property (GObject    *object,
-                               guint       prop_id,
-                               GValue     *value,
-                               GParamSpec *pspec)
-{
-  PpPrinterEntry *self = PP_PRINTER_ENTRY (object);
-
-  switch (prop_id)
-    {
-      case PROP_PRINTER_NAME:
-        g_value_set_string (value, self->printer_name);
-        break;
-      case PROP_PRINTER_LOCATION:
-        g_value_set_string (value, self->printer_location);
-        break;
-      default:
-        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-    }
-}
-
-static void
-pp_printer_entry_set_property (GObject      *object,
-                               guint         prop_id,
-                               const GValue *value,
-                               GParamSpec   *pspec)
-{
-  PpPrinterEntry *self = PP_PRINTER_ENTRY (object);
-
-  switch (prop_id)
-    {
-      case PROP_PRINTER_NAME:
-        self->printer_name = g_value_dup_string (value);
-        break;
-      case PROP_PRINTER_LOCATION:
-        self->printer_location = g_value_dup_string (value);
-        break;
-      default:
-        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-        break;
-    }
-}
-
 static void
 pp_printer_entry_init (PpPrinterEntry *self)
 {
@@ -261,7 +212,7 @@ tone_down_color (GdkRGBA *color,
                  gdouble  saturation_ratio,
                  gdouble  value_ratio)
 {
-  gdouble h, s, v;
+  gfloat h, s, v;
 
   gtk_rgb_to_hsv (color->red, color->green, color->blue,
                   &h, &s, &v);
@@ -269,22 +220,21 @@ tone_down_color (GdkRGBA *color,
                   &color->red, &color->green, &color->blue);
 }
 
-static gboolean
-supply_levels_draw_cb (PpPrinterEntry *self,
-                       cairo_t        *cr)
+static void
+supply_levels_draw_cb (GtkDrawingArea *drawing_area,
+                       cairo_t        *cr,
+                       int             width,
+                       int             height,
+                       gpointer        user_data)
 {
+  PpPrinterEntry         *self = PP_PRINTER_ENTRY (user_data);
   GtkStyleContext        *context;
   gboolean                is_empty = TRUE;
   g_autofree gchar       *tooltip_text = NULL;
-  gint                    width;
-  gint                    height;
   int                     i;
 
   context = gtk_widget_get_style_context (GTK_WIDGET (self->supply_drawing_area));
 
-  width = gtk_widget_get_allocated_width (GTK_WIDGET (self->supply_drawing_area));
-  height = gtk_widget_get_allocated_height (GTK_WIDGET (self->supply_drawing_area));
-
   gtk_render_background (context, cr, 0, 0, width, height);
 
   if (!supply_level_is_empty (self))
@@ -393,8 +343,6 @@ supply_levels_draw_cb (PpPrinterEntry *self,
 
   gtk_widget_set_visible (GTK_WIDGET (self->printer_inklevel_label), !is_empty);
   gtk_widget_set_visible (GTK_WIDGET (self->supply_frame), !is_empty);
-
-  return TRUE;
 }
 
 static void
@@ -403,36 +351,21 @@ on_printer_rename_cb (GObject      *source_object,
                       gpointer      user_data)
 {
   PpPrinterEntry *self = user_data;
-  g_autofree gchar *printer_name = NULL;
 
   if (!pp_printer_rename_finish (PP_PRINTER (source_object), result, NULL))
     return;
 
-  g_object_get (PP_PRINTER (source_object),
-                "printer-name", &printer_name,
-                NULL);
-
-  g_signal_emit_by_name (self, "printer-renamed", printer_name);
+  g_signal_emit_by_name (self, "printer-renamed", pp_printer_get_name (PP_PRINTER (source_object)));
 }
 
 static void
-on_show_printer_details_dialog (GtkButton      *button,
-                                PpPrinterEntry *self)
+show_printer_details_response_cb (PpDetailsDialog *dialog,
+                                  gint             reponse,
+                                  PpPrinterEntry  *self)
 {
   const gchar *new_name;
   const gchar *new_location;
 
-  PpDetailsDialog *dialog = pp_details_dialog_new (self->printer_name,
-                                                   self->printer_location,
-                                                   self->printer_hostname,
-                                                   self->printer_make_and_model,
-                                                   self->is_authorized);
-
-  gtk_window_set_transient_for (GTK_WINDOW (dialog),
-                                GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (self))));
-
-  gtk_dialog_run (GTK_DIALOG (dialog));
-
   new_location = pp_details_dialog_get_printer_location (dialog);
   if (g_strcmp0 (self->printer_location, new_location) != 0)
     printer_set_location (self->printer_name, new_location);
@@ -440,7 +373,7 @@ on_show_printer_details_dialog (GtkButton      *button,
   new_name = pp_details_dialog_get_printer_name (dialog);
   if (g_strcmp0 (self->printer_name, new_name) != 0)
     {
-      PpPrinter *printer = pp_printer_new (self->printer_name);
+      g_autoptr(PpPrinter) printer = pp_printer_new (self->printer_name);
 
       pp_printer_rename_async (printer,
                                new_name,
@@ -451,7 +384,25 @@ on_show_printer_details_dialog (GtkButton      *button,
 
   g_signal_emit_by_name (self, "printer-changed");
 
-  gtk_widget_destroy (GTK_WIDGET (dialog));
+  gtk_window_destroy (GTK_WINDOW (dialog));
+}
+
+static void
+on_show_printer_details_dialog (GtkButton      *button,
+                                PpPrinterEntry *self)
+{
+  PpDetailsDialog *dialog = pp_details_dialog_new (self->printer_name,
+                                                   self->printer_location,
+                                                   self->printer_hostname,
+                                                   self->printer_make_and_model,
+                                                   self->is_authorized);
+
+  gtk_window_set_transient_for (GTK_WINDOW (dialog),
+                                GTK_WINDOW (gtk_widget_get_native (GTK_WIDGET (self))));
+
+  g_signal_connect (dialog, "response", G_CALLBACK (show_printer_details_response_cb), self);
+
+  gtk_window_present (GTK_WINDOW (dialog));
 }
 
 static void
@@ -463,16 +414,14 @@ on_show_printer_options_dialog (GtkButton      *button,
   dialog = pp_options_dialog_new (self->printer_name, self->is_authorized);
 
   gtk_window_set_transient_for (GTK_WINDOW (dialog),
-                                GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (self))));
+                                GTK_WINDOW (gtk_widget_get_native (GTK_WIDGET (self))));
 
-  gtk_dialog_run (GTK_DIALOG (dialog));
-
-  gtk_widget_destroy (GTK_WIDGET (dialog));
+  gtk_window_present (GTK_WINDOW (dialog));
 }
 
 static void
-set_as_default_printer (GtkToggleButton *button,
-                        PpPrinterEntry  *self)
+set_as_default_printer (GtkCheckButton *button,
+                        PpPrinterEntry *self)
 {
   printer_set_default (self->printer_name);
 
@@ -485,24 +434,20 @@ check_clean_heads_maintenance_command_cb (GObject      *source_object,
                                           gpointer      user_data)
 {
   PpPrinterEntry       *self = user_data;
-  PpMaintenanceCommand *command = (PpMaintenanceCommand *) source_object;
   gboolean              is_supported = FALSE;
   g_autoptr(GError)     error = NULL;
 
-  is_supported = pp_maintenance_command_is_supported_finish (command, res, &error);
+  is_supported = pp_maintenance_command_is_supported_finish (PP_MAINTENANCE_COMMAND (source_object), res, &error);
   if (error != NULL)
     {
       g_debug ("Could not check 'Clean' maintenance command: %s", error->message);
-      goto out;
+      return;
     }
 
   if (is_supported)
     {
       gtk_widget_show (GTK_WIDGET (self->clean_heads_menuitem));
     }
-
- out:
-  g_object_unref (source_object);
 }
 
 static void
@@ -525,15 +470,11 @@ clean_heads_maintenance_command_cb (GObject      *source_object,
                                     GAsyncResult *res,
                                     gpointer      user_data)
 {
-  PpPrinterEntry       *self = user_data;
-  PpMaintenanceCommand *command = (PpMaintenanceCommand *) source_object;
-  g_autoptr(GError)     error = NULL;
+  PpPrinterEntry *self = user_data;
+  g_autoptr(GError) error = NULL;
 
-  if (!pp_maintenance_command_execute_finish (command, res, &error))
-    {
-      g_warning ("Error cleaning print heads for %s: %s", self->printer_name, error->message);
-    }
-  g_object_unref (source_object);
+  if (!pp_maintenance_command_execute_finish (PP_MAINTENANCE_COMMAND (source_object), res, &error))
+    g_warning ("Error cleaning print heads for %s: %s", self->printer_name, error->message);
 }
 
 static void
@@ -563,14 +504,11 @@ get_jobs_cb (GObject      *source_object,
              gpointer      user_data)
 {
   PpPrinterEntry      *self = user_data;
-  PpPrinter           *printer = PP_PRINTER (source_object);
   g_autoptr(GError)    error = NULL;
   g_autoptr(GPtrArray) jobs = NULL;
   g_autofree gchar    *button_label = NULL;
 
-  jobs = pp_printer_get_jobs_finish (printer, result, &error);
-
-  g_object_unref (source_object);
+  jobs = pp_printer_get_jobs_finish (PP_PRINTER (source_object), result, &error);
 
   if (error != NULL)
     {
@@ -607,7 +545,7 @@ get_jobs_cb (GObject      *source_object,
 void
 pp_printer_entry_update_jobs_count (PpPrinterEntry *self)
 {
-  PpPrinter *printer;
+  g_autoptr(PpPrinter) printer = NULL;
 
   g_cancellable_cancel (self->get_jobs_cancellable);
   g_clear_object (&self->get_jobs_cancellable);
@@ -632,7 +570,7 @@ jobs_dialog_response_cb (GtkDialog  *dialog,
 
   if (self->pp_jobs_dialog != NULL)
     {
-      gtk_widget_destroy (GTK_WIDGET (self->pp_jobs_dialog));
+      gtk_window_destroy (GTK_WINDOW (self->pp_jobs_dialog));
       self->pp_jobs_dialog = NULL;
     }
 }
@@ -644,7 +582,7 @@ pp_printer_entry_show_jobs_dialog (PpPrinterEntry *self)
     {
       self->pp_jobs_dialog = pp_jobs_dialog_new (self->printer_name);
       g_signal_connect_object (self->pp_jobs_dialog, "response", G_CALLBACK (jobs_dialog_response_cb), self, 0);
-      gtk_window_set_transient_for (GTK_WINDOW (self->pp_jobs_dialog), GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (self))));
+      gtk_window_set_transient_for (GTK_WINDOW (self->pp_jobs_dialog), GTK_WINDOW (gtk_widget_get_native (GTK_WIDGET (self))));
       gtk_window_present (GTK_WINDOW (self->pp_jobs_dialog));
     }
 }
@@ -702,7 +640,9 @@ pp_printer_entry_new (cups_dest_t  printer,
 {
   PpPrinterEntry *self;
 
-  self = g_object_new (PP_PRINTER_ENTRY_TYPE, "printer-name", printer.name, NULL);
+  self = g_object_new (PP_PRINTER_ENTRY_TYPE, NULL);
+
+  self->printer_name = g_strdup (printer.name);
 
   self->clean_command = pp_maintenance_command_new (self->printer_name,
                                                     "Clean",
@@ -711,13 +651,30 @@ pp_printer_entry_new (cups_dest_t  printer,
                                                     _("Clean print heads"));
   check_clean_heads_maintenance_command (self);
 
-  g_signal_connect_object (self->supply_drawing_area, "draw", G_CALLBACK (supply_levels_draw_cb), self, G_CONNECT_SWAPPED);
+  gtk_drawing_area_set_draw_func (self->supply_drawing_area,
+                                  supply_levels_draw_cb,
+                                  self,
+                                  NULL);
 
   pp_printer_entry_update (self, printer, is_authorized);
 
   return self;
 }
 
+const gchar *
+pp_printer_entry_get_name (PpPrinterEntry *self)
+{
+  g_return_val_if_fail (PP_IS_PRINTER_ENTRY (self), NULL);
+  return self->printer_name;
+}
+
+const gchar *
+pp_printer_entry_get_location (PpPrinterEntry *self)
+{
+  g_return_val_if_fail (PP_IS_PRINTER_ENTRY (self), NULL);
+  return self->printer_location;
+}
+
 void
 pp_printer_entry_update (PpPrinterEntry *self,
                          cups_dest_t     printer,
@@ -935,7 +892,8 @@ pp_printer_entry_update (PpPrinterEntry *self,
   else
     printer_icon_name = g_strdup ("printer-network");
 
-  g_object_set (self, "printer-location", location, NULL);
+  g_free (self->printer_location);
+  self->printer_location = g_strdup (location);
 
   self->is_accepting_jobs = is_accepting_jobs;
   self->is_authorized = is_authorized;
@@ -943,11 +901,11 @@ pp_printer_entry_update (PpPrinterEntry *self,
   g_free (self->printer_hostname);
   self->printer_hostname = printer_get_hostname (printer_type, device_uri, printer_uri);
 
-  gtk_image_set_from_icon_name (self->printer_icon, printer_icon_name, GTK_ICON_SIZE_DIALOG);
+  gtk_image_set_from_icon_name (self->printer_icon, printer_icon_name);
   gtk_label_set_text (self->printer_status, printer_status);
   gtk_label_set_text (self->printer_name_label, instance);
   g_signal_handlers_block_by_func (self->printer_default_checkbutton, set_as_default_printer, self);
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->printer_default_checkbutton), printer.is_default);
+  gtk_check_button_set_active (self->printer_default_checkbutton, printer.is_default);
   g_signal_handlers_unblock_by_func (self->printer_default_checkbutton, set_as_default_printer, self);
 
   self->printer_make_and_model = sanitize_printer_model (printer_make_and_model);
@@ -1035,26 +993,8 @@ pp_printer_entry_class_init (PpPrinterEntryClass *klass)
   gtk_widget_class_bind_template_callback (widget_class, show_jobs_dialog);
   gtk_widget_class_bind_template_callback (widget_class, restart_printer);
 
-  object_class->get_property = pp_printer_entry_get_property;
-  object_class->set_property = pp_printer_entry_set_property;
   object_class->dispose = pp_printer_entry_dispose;
 
-  g_object_class_install_property (object_class,
-                                   PROP_PRINTER_NAME,
-                                   g_param_spec_string ("printer-name",
-                                                        "Printer Name",
-                                                        "The Printer unique name",
-                                                        NULL,
-                                                        G_PARAM_READWRITE));
-
-  g_object_class_install_property (object_class,
-                                   PROP_PRINTER_LOCATION,
-                                   g_param_spec_string ("printer-location",
-                                                        "Printer Location",
-                                                        "Printer location string",
-                                                        NULL,
-                                                        G_PARAM_READWRITE));
-
   signals[IS_DEFAULT_PRINTER] =
     g_signal_new ("printer-changed",
                   G_TYPE_FROM_CLASS (klass),
